/*
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 1999, 2000 MIPS Technologies, Inc.  All rights reserved.
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * Interrupt exception dispatch code.
 * HISTORY
 * $Version $Date      $Author     $Comment
 *  2.0.0   27/04/07   Teh Kok How Use unified function calls and defines
 *                                 between Danube and Amazon_SE. Benefits is,
 *                                 of course, better code management.
 */
#include <linux/config.h>

#include <asm/asm.h>
#include <asm/mipsregs.h>
#include <asm/regdef.h>
#include <asm/stackframe.h>
#define COMPAT_IRQ
#ifdef COMPAT_IRQ
	.text
	.set	noreorder
	.set	noat
	.align	5
	NESTED(mipsIRQ, PT_SIZE, sp)
	SAVE_ALL
	CLI
	.set	at

	mfc0	s0, CP0_CAUSE		# get irq mask
	mfc0	t1, CP0_STATUS
	and	s0, t1

	/* First we check for r4k counter/timer IRQ. */
	andi	a0, s0, CAUSEF_IP7
	beq	a0, zero, 1f
	nop

	/* Wheee, a timer interrupt. */
	move	a0, sp
	/* NOTE: this may just acknowledge the interrupt */
	jal	hw5_irqdispatch
	 nop

	j	ret_from_irq
	 nop

1:
	andi	a0, s0, CAUSEF_IP2	# delay slot, check hw0 interrupt
	beq	a0, zero, 1f
	nop
	/* Wheee, combined hardware level zero interrupt. */
	jal	hw0_irqdispatch
	 move	a0, sp			# delay slot

	j	ret_from_irq
	 nop				# delay slot

1:
	andi	a0, s0, CAUSEF_IP3	# delay slot, check hw1 interrupt
	beq	a0, zero, 1f
	nop
	/* Wheee, combined hardware level one interrupt. */
	jal	hw1_irqdispatch
	 move	a0, sp			# delay slot

	j	ret_from_irq
	 nop				# delay slot

1:
	andi	a0, s0, CAUSEF_IP4	# delay slot, check hw2 interrupt
	beq	a0, zero, 1f
	nop
	/* Wheee, combined hardware level two interrupt. */
	jal	hw2_irqdispatch
	 move	a0, sp			# delay slot

	j	ret_from_irq
	 nop				# delay slot
1:
        andi    a0, s0, CAUSEF_IP5      # delay slot, check hw3 interrupt
        beq     a0, zero, 1f
        nop
        /* Wheee, combined hardware level three interrupt. */
        jal     hw3_irqdispatch
         move   a0, sp                  # delay slot
                                                                                                         
        j       ret_from_irq
         nop                            # delay slot
                                                                                                         
1:
        andi    a0, s0, CAUSEF_IP6      # delay slot, check hw4 interrupt
        beq     a0, zero, 1f
        nop
        /* Wheee, combined hardware level three interrupt. */
        jal     hw4_irqdispatch
         move   a0, sp                  # delay slot
                                                                                                         
        j       ret_from_irq
         nop                            # delay slot

1:
	/*
	 * Here by mistake?  This is possible, what can happen is that by the
	 * time we take the exception the IRQ pin goes low, so just leave if
	 * this is the case.
	 */
	move	a1,s0
	PRINT("Got interrupt: c0_cause = %08x\n")
	mfc0	a1, CP0_EPC
	PRINT("c0_epc = %08x\n")

	j	ret_from_irq
	nop
	END(mipsIRQ)
#else
        /* Vectored Interrupt Table: */
        /* Vectored Interrupt Mode is used when the following conditions are met: */
        /* CONFIG3[VInt] = 1 */
        /* CONFIG3[Veic] = 0 */
        /* INTCTL[VS]   != 0 */
        /* CAUSE[IV]     = 1 */
        /* STATUS[BEV]   = 0 */

        .text
        .set    noreorder
        .set    noat
        .align  5
        .section "DanubeIRQ0","ax"
        NESTED(mipsIRQ0, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0)

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ0)

        .globl end_mipsIRQ0
end_mipsIRQ0:


        .set    noat
        .align  5
        .section "DanubeIRQ1","ax"
        NESTED(mipsIRQ1, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1)

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ1)

        .globl end_mipsIRQ1
end_mipsIRQ1:


        .set    noat
        .align  5
        .section "DanubeIRQ2","ax"
        NESTED(mipsIRQ2, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1|IE_IRQ0)

        jal     hw0_irqdispatch
        move    a0, sp                  # load struct pt_regs *regs to a0 in delay slot

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ2)

        .globl end_mipsIRQ2
end_mipsIRQ2:


        .set    noat
        .align  5
        .section "DanubeIRQ3","ax"
        NESTED(mipsIRQ3, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1|IE_IRQ0|IE_IRQ1)

        jal     hw1_irqdispatch
        move    a0, sp                  # load struct pt_regs *regs to a0 in delay slot

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ3)

        .globl end_mipsIRQ3
end_mipsIRQ3:

        .set    noat
        .align  5
        .section "DanubeIRQ4","ax"
        NESTED(mipsIRQ4, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1|IE_IRQ0|IE_IRQ1|IE_IRQ2)

        jal     hw2_irqdispatch
        move    a0, sp                  # load struct pt_regs *regs to a0 in delay slot

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ4)

        .globl end_mipsIRQ4
end_mipsIRQ4:


        .set    noat
        .align  5
        .section "DanubeIRQ5","ax"
        NESTED(mipsIRQ5, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1|IE_IRQ0|IE_IRQ1|IE_IRQ2|IE_IRQ3)

        jal     hw3_irqdispatch
        move    a0, sp                  # load struct pt_regs *regs to a0 in delay slot

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ5)

        .globl end_mipsIRQ5
end_mipsIRQ5:


        .set    noat
        .align  5
        .section "DanubeIRQ6","ax"
        NESTED(mipsIRQ6, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1|IE_IRQ0|IE_IRQ1|IE_IRQ2|IE_IRQ3|IE_IRQ4)

        jal     hw4_irqdispatch
        move    a0, sp                  # load struct pt_regs *regs to a0 in delay slot

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ6)

        .globl end_mipsIRQ6
end_mipsIRQ6:


        .set    noat
        .align  5
        .section "DanubeIRQ7","ax"
        NESTED(mipsIRQ7, PT_SIZE, sp)
        SAVE_ALL
        .set    at
        MASKIRQ(IE_SW0|IE_SW1|IE_IRQ0|IE_IRQ1|IE_IRQ2|IE_IRQ3|IE_IRQ4|IE_IRQ5)

        jal     hw5_irqdispatch
        move    a0, sp

        RESTOREIRQ
        j       ret_from_irq
        nop
        END(mipsIRQ7)

        .globl end_mipsIRQ7
end_mipsIRQ7:
#endif
